use diesel::{
    dsl::{exists, not},
    prelude::*,
};

use crate::schema::*;

#[derive(Queryable, Selectable)]
#[diesel(table_name = sync_groups)]
#[diesel(check_for_backend(diesel::sqlite::Sqlite))]
pub struct SyncGroup {
    pub id: i64,
}

impl SyncGroup {
    pub fn new(conn: &mut SqliteConnection) -> QueryResult<Self> {
        diesel::insert_into(sync_groups::table)
            .default_values()
            .returning(SyncGroup::as_returning())
            .get_result(conn)
    }

    pub fn remove_unused(conn: &mut SqliteConnection) -> QueryResult<usize> {
        diesel::delete(
            sync_groups::table.filter(not(exists(
                devices::table
                    .select(1.into_sql::<diesel::sql_types::Integer>())
                    .filter(devices::sync_group_id.eq(sync_groups::id.nullable())),
            ))),
        )
        .execute(conn)
    }
}
